@@ -1,7 +1,9 @@
 # Copyright 2012 by Wibowo Arindrarto.  All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 
 """Bio.SearchIO parser for BLAST+ XML output formats."""
 # for more info: http://www.ncbi.nlm.nih.gov/dtd/NCBI_BlastOutput.mod.dtd
@@ -33,7 +35,7 @@ from Bio.SearchIO._model import QueryResult, Hit, HSP, HSPFragment
 from Bio._py3k import _as_bytes, _bytes_to_string, unicode
 _empty_bytes_string = _as_bytes("")
 
-__all__ = ['BlastXmlParser', 'BlastXmlIndexer', 'BlastXmlWriter']
+__all__ = ('BlastXmlParser', 'BlastXmlIndexer', 'BlastXmlWriter')
 
 
 # element - optional qresult attribute name mapping
@@ -188,26 +190,46 @@ _RE_ID_DESC_PAIRS_PATTERN = re.compile(" +>")
 _RE_ID_DESC_PATTERN = re.compile(" +")
 
 
-def _extract_ids_and_descs(concat_str):
-    # Given a string space-separate string of IDs and descriptions,
-    # return a list of tuples, each tuple containing an ID and
-    # a description string (which may be empty)
+def _extract_ids_and_descs(raw_id, raw_desc):
+    """Extract IDs, descriptions, and raw ID from raw values (PRIVATE).
+
+    Given values of the `Hit_id` and `Hit_def` elements, this function returns
+    a tuple of three elements: all IDs, all descriptions, and the
+    BLAST-generated ID. The BLAST-generated ID is set to `None` if no
+    BLAST-generated IDs are present.
+
+    """
+    ids = []
+    descs = []
+
+    blast_gen_id = raw_id
+    if raw_id.startswith('gnl|BL_ORD_ID|'):
+        id_desc_line = raw_desc
+    else:
+        id_desc_line = raw_id + ' ' + raw_desc
 
     # create a list of lists, each list containing an ID and description
     # or just an ID, if description is not present
     id_desc_pairs = [re.split(_RE_ID_DESC_PATTERN, x, 1)
-                     for x in re.split(_RE_ID_DESC_PAIRS_PATTERN, concat_str)]
+                     for x in re.split(_RE_ID_DESC_PAIRS_PATTERN, id_desc_line)]
     # make sure empty descriptions are added as empty strings
     # also, we return lists for compatibility reasons between Py2 and Py3
     add_descs = lambda x: x if len(x) == 2 else x + [""]
-    return [pair for pair in map(add_descs, id_desc_pairs)]
+    for pair in (add_descs(p) for p in id_desc_pairs):
+        ids.append(pair[0])
+        descs.append(pair[1])
+
+    return (ids, descs, blast_gen_id)
 
 
 class BlastXmlParser(object):
-    """Parser for the BLAST XML format"""
+    """Parser for the BLAST XML format."""
 
-    def __init__(self, handle):
+    def __init__(self, handle, use_raw_query_ids=False, use_raw_hit_ids=False):
+        """Initialize the class."""
         self.xml_iter = iter(ElementTree.iterparse(handle, events=('start', 'end')))
+        self._use_raw_query_ids = use_raw_query_ids
+        self._use_raw_hit_ids = use_raw_hit_ids
         self._meta, self._fallback = self._parse_preamble()
 
     def __iter__(self):
@@ -215,7 +237,7 @@ class BlastXmlParser(object):
             yield qresult
 
     def _parse_preamble(self):
-        """Parses all tag data prior to the first query result."""
+        """Parse all tag data prior to the first query result (PRIVATE)."""
         # dictionary for containing all information prior to the first query
         meta = {}
         # dictionary for fallback information
@@ -260,7 +282,7 @@ class BlastXmlParser(object):
         return meta, fallback
 
     def _parse_qresult(self):
-        """Parses query results."""
+        """Parse query results (PRIVATE)."""
         # parse the queries
         for event, qresult_elem in self.xml_iter:
             # </Iteration> marks the end of a single query
@@ -290,20 +312,19 @@ class BlastXmlParser(object):
                 if query_len is None:
                     query_len = self._fallback['len']
 
+                blast_query_id = query_id
                 # handle blast searches against databases with Blast's IDs
                 # 'Query_' marks the beginning of a BLAST+-generated ID,
                 # 'lcl|' marks the beginning of a BLAST legacy-generated ID
-                if query_id.startswith('Query_') or query_id.startswith('lcl|'):
+                if not self._use_raw_query_ids and \
+                        (query_id.startswith('Query_') or query_id.startswith('lcl|')):
                     # store the Blast-generated query ID
-                    blast_query_id = query_id
                     id_desc = query_desc.split(' ', 1)
                     query_id = id_desc[0]
                     try:
                         query_desc = id_desc[1]
                     except IndexError:
                         query_desc = ''
-                else:
-                    blast_query_id = ''
 
                 hit_list, key_list = [], []
                 for hit in self._parse_hit(qresult_elem.find('Iteration_hits'),
@@ -311,18 +332,18 @@ class BlastXmlParser(object):
                     if hit:
                         # need to keep track of hit IDs, since there could be duplicates,
                         if hit.id in key_list:
-                            warnings.warn("Adding hit with BLAST-generated ID "
-                                    "%r since hit ID %r is already present "
-                                    "in query %r. Your BLAST database may contain "
+                            warnings.warn("Renaming hit ID %r to a BLAST-generated ID "
+                                    "%r since the ID was already matched "
+                                    "by your query %r. Your BLAST database may contain "
                                     "duplicate entries." %
-                                    (hit._blast_id, hit.id, query_id), BiopythonParserWarning)
+                                    (hit.id, hit.blast_id, query_id), BiopythonParserWarning)
                             # fallback to Blast-generated IDs, if the ID is already present
                             # and restore the desc, too
                             hit.description = '%s %s' % (hit.id, hit.description)
-                            hit.id = hit._blast_id
+                            hit.id = hit.blast_id
                             # and change the hit_id of the HSPs contained
                             for hsp in hit:
-                                hsp.hit_id = hit._blast_id
+                                hsp.hit_id = hit.blast_id
                         else:
                             key_list.append(hit.id)
 
@@ -332,7 +353,7 @@ class BlastXmlParser(object):
                 qresult = QueryResult(hit_list, query_id)
                 qresult.description = query_desc
                 qresult.seq_len = int(query_len)
-                qresult._blast_id = blast_query_id
+                qresult.blast_id = blast_query_id
                 for key, value in self._meta.items():
                     setattr(qresult, key, value)
 
@@ -365,7 +386,7 @@ class BlastXmlParser(object):
                 yield qresult
 
     def _parse_hit(self, root_hit_elem, query_id):
-        """Generator that transforms Iteration_hits XML elements into Hit objects.
+        """Yield a generator object that transforms Iteration_hits XML elements into Hit objects (PRIVATE).
 
         :param root_hit_elem: root element of the Iteration_hits tag.
         :type root_hit_elem: XML element tag
@@ -390,25 +411,17 @@ class BlastXmlParser(object):
 
         for hit_elem in root_hit_elem:
 
-            # create empty hit object
-            hit_id = hit_elem.findtext('Hit_id')
-            hit_desc = hit_elem.findtext('Hit_def')
-            # handle blast searches against databases with Blast's IDs
-            if hit_id.startswith('gnl|BL_ORD_ID|'):
-                blast_hit_id = hit_id
-                id_desc = hit_desc.split(' ', 1)
-                hit_id = id_desc[0]
-                try:
-                    hit_desc = id_desc[1]
-                except IndexError:
-                    hit_desc = ''
+            # BLAST sometimes mangles the sequence IDs and descriptions, so we need
+            # to extract the actual values.
+            raw_hit_id = hit_elem.findtext('Hit_id')
+            raw_hit_desc = hit_elem.findtext('Hit_def')
+            if not self._use_raw_hit_ids:
+                ids, descs, blast_hit_id = _extract_ids_and_descs(raw_hit_id, raw_hit_desc)
             else:
-                blast_hit_id = ''
+                ids, descs, blast_hit_id = [raw_hit_id], [raw_hit_desc], raw_hit_id
 
-            # combine primary ID and defline first before splitting
-            full_id_desc = hit_id + ' ' + hit_desc
-            id_descs = _extract_ids_and_descs(full_id_desc)
-            hit_id, hit_desc = id_descs[0]
+            hit_id, alt_hit_ids = ids[0], ids[1:]
+            hit_desc, alt_hit_descs = descs[0], descs[1:]
 
             hsps = [hsp for hsp in
                     self._parse_hsp(hit_elem.find('Hit_hsps'),
@@ -416,10 +429,9 @@ class BlastXmlParser(object):
 
             hit = Hit(hsps)
             hit.description = hit_desc
-            hit._id_alt = [x[0] for x in id_descs[1:]]
-            hit._description_alt = [x[1] for x in id_descs[1:]]
-            # blast_hit_id is only set if the hit ID is Blast-generated
-            hit._blast_id = blast_hit_id
+            hit._id_alt = alt_hit_ids
+            hit._description_alt = alt_hit_descs
+            hit.blast_id = blast_hit_id
 
             for key, val_info in _ELEM_HIT.items():
                 value = hit_elem.findtext(key)
@@ -435,7 +447,7 @@ class BlastXmlParser(object):
             yield hit
 
     def _parse_hsp(self, root_hsp_frag_elem, query_id, hit_id):
-        """Iterator that transforms Hit_hsps XML elements into HSP objects.
+        """Yield a generator object that transforms Hit_hsps XML elements into HSP objects (PRIVATE).
 
         :param root_hsp_frag_elem: the ``Hit_hsps`` tag
         :type root_hsp_frag_elem: XML element tag
@@ -491,8 +503,7 @@ class BlastXmlParser(object):
                     setattr(frag, val_info[0], value)
 
             # set the similarity characters into aln_annotation dict
-            frag.aln_annotation['similarity'] = \
-                    hsp_frag_elem.findtext('Hsp_midline')
+            frag.aln_annotation['similarity'] = hsp_frag_elem.findtext('Hsp_midline')
 
             # process coordinates
             # since 'x-from' could be bigger than 'x-to', we need to figure
@@ -539,10 +550,11 @@ class BlastXmlIndexer(SearchIndexer):
     qend_mark = _as_bytes('</Iteration>')
     block_size = 16384
 
-    def __init__(self, filename):
+    def __init__(self, filename, **kwargs):
+        """Initialize the class."""
         SearchIndexer.__init__(self, filename)
         # TODO: better way to do this?
-        iter_obj = self._parser(self._handle)
+        iter_obj = self._parser(self._handle, **kwargs)
         self._meta, self._fallback = iter_obj._meta, iter_obj._fallback
 
     def __iter__(self):
@@ -645,13 +657,13 @@ class _BlastXmlGenerator(XMLGenerator):
             self.write = self._out.write
 
     def startDocument(self):
-        """Starts the XML document."""
+        """Start the XML document."""
         self.write(u'<?xml version="1.0"?>\n'
                 '<!DOCTYPE BlastOutput PUBLIC "-//NCBI//NCBI BlastOutput/EN" '
                 '"http://www.ncbi.nlm.nih.gov/dtd/NCBI_BlastOutput.dtd">\n')
 
     def startElement(self, name, attrs=None, children=False):
-        """Starts an XML element.
+        """Start an XML element.
 
         :param name: element name
         :type name: string
@@ -667,12 +679,12 @@ class _BlastXmlGenerator(XMLGenerator):
         XMLGenerator.startElement(self, name, attrs)
 
     def endElement(self, name):
-        """Ends and XML element of the given name."""
+        """End and XML element of the given name."""
         XMLGenerator.endElement(self, name)
         self.write(u'\n')
 
     def startParent(self, name, attrs=None):
-        """Starts an XML element which has children.
+        """Start an XML element which has children.
 
         :param name: element name
         :type name: string
@@ -689,7 +701,7 @@ class _BlastXmlGenerator(XMLGenerator):
         self._parent_stack.append(name)
 
     def endParent(self):
-        """Ends an XML element with children."""
+        """End an XML element with children."""
         # the element to end is the one on top of the stack
         name = self._parent_stack.pop()
         self._level -= self._increment
@@ -697,17 +709,17 @@ class _BlastXmlGenerator(XMLGenerator):
         self.endElement(name)
 
     def startParents(self, *names):
-        """Starts XML elements without children."""
+        """Start XML elements without children."""
         for name in names:
             self.startParent(name)
 
     def endParents(self, num):
-        """Ends XML elements, according to the given number."""
+        """End XML elements, according to the given number."""
         for i in range(num):
             self.endParent()
 
     def simpleElement(self, name, content=None):
-        """Creates an XML element without children with the given content."""
+        """Create an XML element without children with the given content."""
         self.startElement(name, attrs={})
         if content:
             self.characters(content)
@@ -723,11 +735,14 @@ class _BlastXmlGenerator(XMLGenerator):
 class BlastXmlWriter(object):
     """Stream-based BLAST+ XML Writer."""
 
-    def __init__(self, handle):
+    def __init__(self, handle, use_raw_query_ids=True, use_raw_hit_ids=True):
+        """Initialize the class."""
         self.xml = _BlastXmlGenerator(handle, 'utf-8')
+        self._use_raw_query_ids = use_raw_query_ids
+        self._use_raw_hit_ids = use_raw_hit_ids
 
     def write_file(self, qresults):
-        """Writes the XML contents to the output handle."""
+        """Write the XML contents to the output handle."""
         xml = self.xml
         self.qresult_counter, self.hit_counter, self.hsp_counter, \
             self.frag_counter = 0, 0, 0, 0
@@ -748,7 +763,7 @@ class BlastXmlWriter(object):
             self.frag_counter
 
     def _write_elem_block(self, block_name, map_name, obj, opt_dict=None):
-        """Writes sibling XML elements.
+        """Write sibling XML elements (PRIVATE).
 
         :param block_name: common element name prefix
         :type block_name: string
@@ -777,7 +792,7 @@ class BlastXmlWriter(object):
                 self.xml.simpleElement(elem, content)
 
     def _write_preamble(self, qresult):
-        """Writes the XML file preamble."""
+        """Write the XML file preamble (PRIVATE)."""
         xml = self.xml
 
         for elem, attr in _WRITE_MAPS['preamble']:
@@ -796,37 +811,40 @@ class BlastXmlWriter(object):
                 if elem == 'BlastOutput_version':
                     content = '%s %s' % (qresult.program.upper(),
                             qresult.version)
-                elif qresult._blast_id:
+                elif qresult.blast_id:
                     if elem == 'BlastOutput_query-ID':
-                        content = qresult._blast_id
+                        content = qresult.blast_id
                     elif elem == 'BlastOutput_query-def':
                         content = ' '.join([qresult.id,
                             qresult.description]).strip()
                 xml.simpleElement(elem, content)
 
     def _write_param(self, qresult):
-        """Writes the parameter block of the preamble."""
+        """Write the parameter block of the preamble (PRIVATE)."""
         xml = self.xml
         xml.startParent('Parameters')
         self._write_elem_block('Parameters_', 'param', qresult)
         xml.endParent()
 
     def _write_qresults(self, qresults):
-        """Writes QueryResult objects into iteration elements."""
+        """Write QueryResult objects into iteration elements (PRIVATE)."""
         xml = self.xml
 
         for num, qresult in enumerate(qresults):
             xml.startParent('Iteration')
             xml.simpleElement('Iteration_iter-num', str(num + 1))
             opt_dict = {}
-            # use custom Iteration_query-ID and Iteration_query-def mapping
-            # if the query has a BLAST-generated ID
-            if qresult._blast_id:
-                opt_dict = {
-                    'Iteration_query-ID': qresult._blast_id,
-                    'Iteration_query-def': ' '.join([qresult.id,
-                            qresult.description]).strip(),
-                }
+            if self._use_raw_query_ids:
+                query_id = qresult.blast_id
+                query_desc = qresult.id + ' ' + qresult.description
+            else:
+                query_id = qresult.id
+                query_desc = qresult.description
+
+            opt_dict = {
+                'Iteration_query-ID': query_id,
+                'Iteration_query-def': query_desc,
+            }
             self._write_elem_block('Iteration_', 'qresult', qresult, opt_dict)
             # the Iteration_hits tag only has children if there are hits
             if qresult:
@@ -847,7 +865,7 @@ class BlastXmlWriter(object):
             xml.endParent()
 
     def _write_hits(self, hits):
-        """Writes Hit objects."""
+        """Write Hit objects (PRIVATE)."""
         xml = self.xml
 
         for num, hit in enumerate(hits):
@@ -856,11 +874,22 @@ class BlastXmlWriter(object):
             # use custom hit_id and hit_def mapping if the hit has a
             # BLAST-generated ID
             opt_dict = {}
-            if hit._blast_id:
-                opt_dict = {
-                    'Hit_id': hit._blast_id,
-                    'Hit_def': ' '.join([hit.id, hit.description]).strip(),
-                }
+
+            if self._use_raw_hit_ids:
+                hit_id = hit.blast_id
+                hit_desc = ' >'.join(
+                    ['{} {}'.format(x, y)
+                     for x, y in zip(hit.id_all, hit.description_all)])
+            else:
+                hit_id = hit.id
+                hit_desc = hit.description + ' >'.join(
+                    ['{} {}'.format(x, y)
+                     for x, y in zip(hit.id_all[1:], hit.description_all[1:])])
+
+            opt_dict = {
+                'Hit_id': hit_id,
+                'Hit_def': hit_desc,
+            }
             self._write_elem_block('Hit_', 'hit', hit, opt_dict)
             xml.startParent('Hit_hsps')
             self._write_hsps(hit.hsps)
@@ -868,7 +897,7 @@ class BlastXmlWriter(object):
             xml.endParents(2)
 
     def _write_hsps(self, hsps):
-        """Writes HSP objects."""
+        """Write HSP objects (PRIVATE)."""
         xml = self.xml
         for num, hsp in enumerate(hsps):
             xml.startParent('Hsp')
@@ -889,8 +918,7 @@ class BlastXmlWriter(object):
             xml.endParent()
 
     def _adjust_output(self, hsp, elem, attr):
-        """Adjusts output to mimic native BLAST+ XML as much as possible."""
-
+        """Adjust output to mimic native BLAST+ XML as much as possible (PRIVATE)."""
         # adjust coordinates
         if attr in ('query_start', 'query_end', 'hit_start', 'hit_end',
                 'pattern_start', 'pattern_end'):
