@@ -1,8 +1,10 @@
 # Copyright 2012 by Wibowo Arindrarto.  All rights reserved.
-# All rights reserved.
-# This code is part of the Biopython distribution and governed by its
-# license.  Please see the LICENSE file that should have been included
-# as part of this package.
+# Revisions copyright 2012-2016 by Peter Cock.  All rights reserved.
+#
+# This file is part of the Biopython distribution and governed by your
+# choice of the "Biopython License Agreement" or the "BSD 3-Clause License".
+# Please see the LICENSE file that should have been included as part of this
+# package.
 
 """Custom indexing for Bio.SearchIO objects."""
 
@@ -20,6 +22,7 @@ class SearchIndexer(_IndexedSeqFileProxy):
     """
 
     def __init__(self, filename, **kwargs):
+        """Initialize the class."""
         self._handle = _open_for_random_access(filename)
         self._kwargs = kwargs
 
